/*
 * Copyright (c) 2003,2004 Aly Merchant
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *     1. The origin of this software must not be misrepresented; you must not
 *        claim that you wrote the original software. If you use this software
 *        in a product, an acknowledgment in the product documentation would be
 *        appreciated but is not required.
 *
 *     2. Altered source versions must be plainly marked as such, and must not
 *        be misrepresented as being the original software.
 *
 *     3. This notice may not be removed or altered from any source
 *        distribution.
 */

package tenk;

import javax.swing.*;

import java.util.ArrayList;

import java.awt.print.*;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

public class Cert extends JPanel implements Printable
{
  private ArrayList lines;

  public class CentredLine
  {
    String text;
    double pos;
    Font   font;

    CentredLine(String t, double p, Font f)
    { text=t; pos=p; font=f; }

    public void draw(Graphics g, int w, int h, int xo, int yo)
    {
      int x,y;
      g.setFont(font);
      FontMetrics fm = g.getFontMetrics();
      x = (w - fm.stringWidth(text))/2 + xo;
      y = (int)Math.round( pos*h ) + yo;
      g.drawString( text, x, y );
    }
  }

  public Cert()
  {
    lines = new ArrayList();
    makeDefaults();
  }

  public Cert(FunRun fr)
  {
    lines = new ArrayList();
    fr.makeCert(this);
  }

  public void addLine(String t, double p, Font f)
  { lines.add( new CentredLine(t,p,f) ); }

  public void makeDefaults()
  {
    addLine("This is to certify that",750d/6800,new Font("Serif",Font.PLAIN,20));
    addLine("Completed the Ismaili 10K Run in a time of",4500d/6800,new Font("Serif",Font.PLAIN,20));
  }
  
  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    drawCert(g,getWidth(),getHeight(),0,0);
  }

  public int print(Graphics g, PageFormat pf, int pi) throws PrinterException 
  {
    if (pi != 0)
      return Printable.NO_SUCH_PAGE;

    System.out.println("Page["+pi+"] format: "+pf.getImageableWidth()+"x"
                        +pf.getImageableHeight()+"+"+pf.getImageableX()+"+"
                        +pf.getImageableY());
    drawCert(g, (int)Math.round(pf.getImageableWidth()),
                (int)Math.round(pf.getImageableHeight()),
                (int)Math.round(pf.getImageableX()),
                (int)Math.round(pf.getImageableY()));
    return Printable.PAGE_EXISTS;
  }

  public void drawCert(Graphics g, int w, int h, int xo, int yo)
  {
    g.setColor(new Color(0));
    for (int i=0; i<lines.size(); i++)
      ((CentredLine)lines.get(i)).draw(g,w,h,xo,yo);
  }
}
