/*
 * Copyright (c) 2003,2004 Aly Merchant
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *     1. The origin of this software must not be misrepresented; you must not
 *        claim that you wrote the original software. If you use this software
 *        in a product, an acknowledgment in the product documentation would be
 *        appreciated but is not required.
 *
 *     2. Altered source versions must be plainly marked as such, and must not
 *        be misrepresented as being the original software.
 *
 *     3. This notice may not be removed or altered from any source
 *        distribution.
 */

package tenk;

import java.io.*;
import java.util.*;
import java.sql.*;

public class Data
{
  private static String dbServer = null,
                        dbName   = null,
                        dbUser   = null,
                        dbPass   = null,
                        dbType   = null,
                        dbDriver = null;
  private static int    dbPort   = -1;
  private static boolean usingDB = true;

  private static Properties p = null;

  private static Connection c = null;

  public static boolean getUsingDB() { return usingDB; }
  public static void setUsingDB(boolean u) { usingDB = u; }

  public static boolean isConnected() { return c != null; }
  public static void setDB() { setDB("db.cfg"); }
  public static void setDB( String propFile )
  {
    
    try
    {
      FileInputStream fis = new FileInputStream(propFile);
      p = new Properties();
      p.load( fis );
      dbServer = p.getProperty("server");
      dbName   = p.getProperty("db");
      dbUser   = p.getProperty("user");
      dbPass   = p.getProperty("pass");
      dbDriver = p.getProperty("driver");
      dbType   = p.getProperty("type");
      dbPort   = (p.getProperty("port") == null) ? -1 : Integer.parseInt(p.getProperty("port"));
    }
    catch (IOException ioe)
    {
      System.out.println("Problem in file reading: "+ioe.getMessage());
      ioe.printStackTrace();
    }
  }

  private static HashMap mr = new HashMap(), fr = new HashMap();

  public static String sqlify(String s)
  {
    for (int i=0; i<s.length(); i++)
      if (s.charAt(i) == '\'')
        s = s.substring(0,i)+"'"+s.substring(i++);
    return s;
  }

  public static void refresh()
  {
    try
    {
      Class.forName(dbDriver).newInstance();
      c = DriverManager.getConnection( "jdbc:"+dbType+"://" + dbServer 
                                       + ((dbPort == -1) ? "" : (":" + dbPort)) 
                                       + "/" + dbName, dbUser,dbPass);
    } 
    catch (Exception e) 
    {
      e.printStackTrace();
      c = null;
      return;
    }
    fillFR();
    fillMR();
  }
 
  public static Map getMRMap() { return mr; }
  public static Map getFRMap() { return fr; }

  public static Object[] getMArr() 
  { Object[] o = mr.values().toArray(); Arrays.sort(o); return o; }
  public static Object[] getFArr()
  { Object[] o = fr.values().toArray(); Arrays.sort(o); return o; }

  public static boolean doUpdate( String sql )
  {
    if (!usingDB) return true;
    if (!isConnected()) return false;
    Statement st = null;
    boolean result = true;
    try
    {
      st = c.createStatement();
      st.executeUpdate(sql);
    }
    catch (SQLException se)
    {
      System.out.println("Couldn't update DB: "+se);
      se.printStackTrace();
      return false;
    }
    finally
    { try { if (st != null) st.close(); } catch (Exception e) {} }
    return true;
  }

  private static String mrInsSQL(MainRun mr)
  {
    return "insert into "+dbName+".mainrun values ('"+sqlify(mr.fname)+"','"+sqlify(mr.lname)
                                             +"',"+mr.prints+"," +mr.bib+","+mr.time
                                             +","+mr.age+",'"+mr.gender+"')";
  }
  private static String frInsSQL(FunRun fr)
  {
    return "insert into "+dbName+".funrun values ('"+sqlify(fr.fname)+"','"
                                            +sqlify(fr.lname)+"',"+fr.prints+")";
  }
  private static String mrDelSQL(MainRun mr)
  {
    return "delete from "+dbName+".mainrun where bib="+mr.bib;
  }
  private static String frDelSQL(FunRun fr)
  {
    return "delete from "+dbName+".funrun where firstname='"+sqlify(fr.fname)
           +"' and lastname='"+sqlify(fr.lname)+"'";
  }
  private static String mrPrnSQL(MainRun mr)
  {
    return "update "+dbName+".mainrun set prints=prints+1 where bib="+mr.bib;
  }
  private static String frPrnSQL(FunRun fr)
  {
    return "update "+dbName+".funrun set prints=prints+1 where firstname='"
           +sqlify(fr.fname)+"' and lastname='"+sqlify(fr.lname)+"'";
  }

  public static boolean addMR( MainRun m )
  {
    if (mr.containsKey( new Integer(m.bib)))
      return false;
    if (!doUpdate(mrInsSQL(m)))
      return false;
    mr.put( new Integer(m.bib), m );
    return true;
  }

  public static boolean delMR( MainRun m )
  {
    if (!mr.containsKey( new Integer(m.bib)))
      return false;
    if (!doUpdate(mrDelSQL(m)))
      return false;
    mr.remove( new Integer(m.bib) );
    return true;
  }

  public static boolean prnMR( MainRun m )
  {
    if (!mr.containsKey( new Integer(m.bib)))
      return false;
    if (!doUpdate(mrPrnSQL(m)))
      return false;
    m.prints++;
    return true;
  }

  public static boolean addFR( FunRun f )
  {
    if (fr.containsKey(f.getFullName()))
      return false;
    if (!doUpdate(frInsSQL(f)))
      return false;
    fr.put( f.getFullName(), f );
    return true;
  }

  public static boolean delFR( FunRun f )
  {
    if (!fr.containsKey( f.getFullName()))
      return false;
    if (!doUpdate(frDelSQL(f)))
      return false;
    fr.remove( f.getFullName() );
    return true;
  }

  public static boolean prnFR( FunRun f )
  {
    if (!fr.containsKey( f.getFullName()))
      return false;
    if (!doUpdate(frPrnSQL(f)))
      return false;
    f.prints++;
    return true;
  }

  public static void fillRunner( FunRun r, ResultSet rs ) throws SQLException
  {
    r.prints = rs.getInt("prints");
    r.fname = rs.getString("firstname");
    r.lname = rs.getString("lastname");
    if (r instanceof MainRun)
    {
      MainRun mr = (MainRun)r;
      mr.time    = rs.getInt("time");
      mr.bib     = rs.getInt("bib");
      mr.age     = rs.getInt("age");
      mr.gender  = rs.getString("gender").toUpperCase().charAt(0);
    }
  }

  private static void fillFR()
  {
    Statement st = null; ResultSet rs = null;
    try
    {
      st = c.createStatement();
      rs = st.executeQuery("select * from "+dbName+".funrun");
      while (rs.next())
      {
        FunRun r = new FunRun();
        fillRunner( r, rs );
        fr.put(r.getFullName(),r);
      }
    }
    catch (SQLException se)
    {
      System.out.println("Fun runners retrieval was not successful: "+se);
      se.printStackTrace();
    }
    finally
    { try { if (rs != null) rs.close(); if (st != null) st.close();} catch (Exception e) {} }
  }

  private static void fillMR()
  {
    Statement st = null; ResultSet rs = null;
    try
    {
      st = c.createStatement();
      rs = st.executeQuery("select * from "+dbName+".mainrun");
      while (rs.next())
      {
        MainRun r = new MainRun();
        fillRunner( r, rs );
        mr.put(new Integer(r.bib),r);
      }
    }
    catch (SQLException se)
    {
      System.out.println("Main runners retrieval was not successful: "+se);
      se.printStackTrace();
    }
    finally
    { try { if (rs != null) rs.close(); if (st != null) st.close();} catch (Exception e) {} }
  }

  public static void main(String[] args) throws Exception
  {
    setDB();
    refresh();
    System.out.println("Am I connected: "+isConnected());
    Object[] f = getFArr();
    for (int i=0; i<f.length; i++)
      System.out.println((FunRun)f[i]);
  }
}
