/*
 * Copyright (c) 2003,2004 Aly Merchant
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *     1. The origin of this software must not be misrepresented; you must not
 *        claim that you wrote the original software. If you use this software
 *        in a product, an acknowledgment in the product documentation would be
 *        appreciated but is not required.
 *
 *     2. Altered source versions must be plainly marked as such, and must not
 *        be misrepresented as being the original software.
 *
 *     3. This notice may not be removed or altered from any source
 *        distribution.
 */

package tenk;

import java.awt.Font;
import javax.swing.SwingUtilities;

public class MainRun extends FunRun
{
  private static final int LNAME=2, FNAME=1, BIB=0, AGE=3, GENDER=4, TIME=5;
  int  time, bib, age;
  char gender;

  public MainRun() { super(); }
  public MainRun(String[] s) { super(s); }
  
  public MainRun (String f, String l, char g, int t, int b, int p, int a)
  { super(f,l,p); time=t; bib=b; age=a; gender=g; }

  public MainRun (String f, String l, char g, String t, int b, int p, int a)
  { super(f,l,p); time=convert(t); bib=b; age=a; gender=g; }

  public MainRun (String big)
  {
    String[] s = big.split(",");
    initFromStrings(s);
  }

  public void initFromStrings(String[] s)
  {
    fname = s[FNAME].trim();
    lname = s[LNAME].trim();
    bib  = Integer.parseInt(s[BIB].trim());
    age  = Integer.parseInt(s[AGE].trim());
    gender = s[GENDER].trim().charAt(0);
    time = convert(s[TIME].trim());
    prints = 0;
  }

  public int compareTo( Object other )
  {
    if (!(other instanceof FunRun)) return 0;
    return getITime().compareTo(((MainRun)other).getITime());
  }

  public void makeCert(Cert c)
  {
    c.addLine("This is to certify that",750d/6800,new Font("Verdana",Font.PLAIN,16));
    c.addLine(getFullName(),1300d/6800,new Font("Times New Roman",Font.ITALIC,36));
    c.addLine("Completed the Ismaili 10K Run in a time of",4400d/6800,new Font("Verdana",Font.PLAIN,16));
    c.addLine(convert(time),5000d/6800,new Font("Verdana",Font.BOLD,24));
    c.addLine("June 20, 2004",5600d/6800,new Font("Times New Roman",Font.PLAIN,16));
  }

  public int getTime() { return time; }
  public Integer getITime() { return new Integer(time); }
  public String getSTime() { return convert(time); }

  private int convert(String t)
  {
    int result=0,mode=1;
    t.trim();
    int pos = -1;
    while ( (pos=t.lastIndexOf(":")) > -1)
    {
      result += Integer.parseInt( t.substring(pos+1) )*mode;
      t = t.substring(0,pos);
      mode *= 60;
    }
    result += Integer.parseInt( t ) * mode;
    return result;
  }


  private String convert(int t)
  {
    String result = (t < 1) ? "00" : "";
    while (t > 0)
    {
      if ( (t % 60) > 9)
        result = ""+ (t % 60) +":"+result;
      else
        result = "0"+ (t % 60) +":"+result;

      t /= 60;
    }
    if (result.length() > 0 && result.charAt(0) == '0')
      result = result.substring(1);
    if (result.length() > 0 && result.charAt(result.length()-1) == ':')
      result = result.substring(0,result.length()-1);
    return result;
  }

  public String toString() 
  { return getFullName()+" ("+bib+") "+convert(time)+" ["+prints+"]"; }

  public static void main(String[] args)
  {
    SwingUtilities.invokeLater( new MainRun(args) );
  }
}
